{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block form_id %}update_domain_form{% endblock %}
{% block form_action %}{% url 'horizon:project:dns_domains:update_domain' domain.id %}{% endblock %}

{% block modal-header %}{% trans "Update Domain" %}{% endblock %}

{% block modal-body %}
  <div class="left">
    <fieldset>
    {% include "horizon/common/_form_fields.html" %}
    </fieldset>
  </div>

  <div class="right">
    <h3>{% trans "Description" %}:</h3>
    <p>{% blocktrans %}
      From here you can edit the email address and TTL associated with a domain.
    {% endblocktrans %}</p>
    <p>{% blocktrans %}
      The Email field should contain a valid email address to be associated
      with the domain.
    {% endblocktrans %}</p>
    <p>{% blocktrans %}
      The optional TTL field can be any value between 1 and 2147483647
      seconds.
    {% endblocktrans %}</p>
  </div>
{% endblock %}

{% block modal-footer %}
  <input class="btn btn-primary pull-right" type="submit" value="{% trans "Update Domain" %}" />
  <a href="{% url 'horizon:project:dns_domains:index' %}" class="btn secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
